/*
DS3231_test.pde
Eric Ayars
4/11

Test/demo of read routines for a DS3231 RTC.

Turn on the serial monitor after loading this to check if things are
working as they should.

Jan Moeskops - 12-06-2019
Deze sketch is een samenvoeging van verschillende opdrachten.  (zie volgende regels welke libraries geladen worden)

*/

#include <LiquidCrystal_I2C.h>
#include <DS3231.h>
#include <BH1750.h>
#include <Wire.h>
#include <DHT.h>

// Set the LCD address to 0x27 for a 16 chars and 2 line display
// LiquidCrystal_I2C lcd(0x27, 16, 2);
LiquidCrystal_I2C lcd(0x3F, 20, 4);    // we gebruiken een 2004 display

BH1750 lightMeter;

#define DHTPIN 2     // Digital pin connected to the DHT sensor
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321
// Initialize DHT sensor.
// Note that older versions of this library took an optional third parameter to
// tweak the timings for faster processors.  This parameter is no longer needed
// as the current DHT reading algorithm adjusts itself to work on faster procs.
DHT dht(DHTPIN, DHTTYPE);

byte graden[8] = {
  0b01000,
  0b10100,
  0b10100,
  0b01000,
  0b00000,
  0b00000,
  0b00000,
  0b00000
};

#define DHTPIN 2     // Digital pin connected to the DHT sensor

DS3231 Clock;
bool Century=false;
bool h12;
bool PM;
byte ADay, AHour, AMinute, ASecond, ABits;
bool ADy, A12h, Apm;

void setup()  // --------------------------------------------- S E T U P --------------------------
{
   // Initialize the I2C bus (BH1750 library doesn't do this automatically)
  Wire.begin();
  lightMeter.begin();   

  dht.begin();

      
    // initialize the LCD
  lcd.init();


  // Turn on the blacklight and print a message.
  lcd.backlight();
  lcd.setCursor(5, 1);

    //lcd.print("Datalogger");
  char titel[10] = {'D','a','t','a','l','o','g','g','e','r'};
  for(int i = 0; i < 10; i++)
  {
    lcd.print(titel[i]);
    delay(50);
  }

lcd.createChar(0, graden);
  
  delay(2000);
  lcd.clear();
  
	// Start the I2C interface
	Wire.begin();
	// Start the serial interface
	Serial.begin(9600);
}

void loop() // --------------------------------------------- L O O P --------------------------
{
  lcd.setCursor(0, 0);
  if (Clock.getDate()<10)
    lcd.print("0");
  lcd.print(Clock.getDate(), DEC);
  lcd.print("/");
  if (Clock.getMonth(Century)<10)
    lcd.print("0");
  
  lcd.print(Clock.getMonth(Century), DEC);
  lcd.print("/");

  lcd.print(Clock.getYear(), DEC);

  lcd.print("  ");
//  lcd.setCursor(0, 1);
  if (Clock.getHour(h12, PM)<10)
    lcd.print("0");
  lcd.print(Clock.getHour(h12, PM), DEC);
  lcd.print(':');
  if (Clock.getMinute()<10)
    lcd.print("0");  
  lcd.print(Clock.getMinute(), DEC);
  lcd.print(':');
  if (Clock.getSecond()<10)
    lcd.print("0");   
  lcd.print(Clock.getSecond(), DEC);
  
	// send what's going on to the serial monitor.

  // then the date
  Serial.print(Clock.getDate(), DEC);
  Serial.print('/');
  // then the month
  Serial.print(Clock.getMonth(Century), DEC);
  Serial.print('/');

  
	// Start with the year


  
//	Serial.print("2");
//	if (Century) {			// Won't need this for 89 years.
//		Serial.print("1");
//	} else {
//		Serial.print("0");
//	}

  Serial.print(Clock.getYear(), DEC);
  Serial.print("     Dag van de week: ");



  
	// and the day of the week
	Serial.print(Clock.getDoW(), DEC);
	Serial.print("  --- ");
  
	// Finally the hour, minute, and second
	Serial.print(Clock.getHour(h12, PM), DEC);
	Serial.print(':');
	Serial.print(Clock.getMinute(), DEC);
	Serial.print(':');
	Serial.print(Clock.getSecond(), DEC);
	// Add AM/PM indicator
	if (h12) {
		if (PM) {
			Serial.print(" PM ");
		} else {
			Serial.print(" AM ");
		}
	} else {
		Serial.print(" 24h ");
	}
	// Display the temperature
	Serial.print("T=");
	Serial.print(Clock.getTemperature(), 2);
	// Tell whether the time is (likely to be) valid
	if (Clock.oscillatorCheck()) {
		Serial.print(" O+");
	} else {
		Serial.print(" O-");
	}
  lcd.setCursor(0, 1);
  lcd.print("RTCtemp:");   // temperatuur op RTC DS3231     RealTimeClock
  lcd.print(Clock.getTemperature(), 2);
  lcd.write(byte(0)); // when calling lcd.write() '0' must be cast as a byte  Print symbool graden
  lcd.print("C ");

  
  float lux = lightMeter.readLightLevel();
  Serial.print("Light: ");
  Serial.print(lux);
  Serial.println(" lx");
  lcd.setCursor(0, 2);
  lcd.print("L:");
  lcd.print(lux);
  lcd.print(" lx");



  // Reading temperature or humidity takes about 250 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  float h = dht.readHumidity();
  // Read temperature as Celsius (the default)
  float t = dht.readTemperature();
  // Read temperature as Fahrenheit (isFahrenheit = true)
  float f = dht.readTemperature(true);

  // Check if any reads failed and exit early (to try again).
  if (isnan(h) || isnan(t) || isnan(f)) {
    Serial.println(F("Failed to read from DHT sensor!"));
    return;
  }

  // Compute heat index in Fahrenheit (the default)
  float hif = dht.computeHeatIndex(f, h);
  // Compute heat index in Celsius (isFahreheit = false)
  float hic = dht.computeHeatIndex(t, h, false);

  Serial.print(F("Humidity: "));
  Serial.print(h);
  Serial.print(F("%  Temperature: "));
  Serial.print(t);
  Serial.print(F("°C "));
  Serial.print(f);
  Serial.print(F("°F  Heat index: "));
  Serial.print(hic);
  Serial.print(F("°C "));
  Serial.print(hif);
  Serial.println(F("°F"));

  lcd.setCursor(0, 3);
  lcd.print("T");
  lcd.print(t);
  lcd.write(byte(0)); // when calling lcd.write() '0' must be cast as a byte  Print symbool graden
  lcd.print("C H");
  lcd.print(h);
  lcd.print(" %  ");
  
  
 
 
	Serial.print('\n');
	delay(1000);
}
