/*
  SD card datalogger

 This example shows how to log data from three analog sensors
 to an SD card using the SD library.

 The circuit:
 * analog sensors on analog ins 0, 1, and 2
 * SD card attached to SPI bus as follows:
 ** MOSI - pin 11
 ** MISO - pin 12
 ** CLK - pin 13
 ** CS - pin 4 (for MKRZero SD: SDCARD_SS_PIN)

 created  24 Nov 2010
 modified 9 Apr 2012
 by Tom Igoe

 This example code is in the public domain.

 Jan Moeskops - 08/06/2019
 - Variabele teller toegevoegd
 - delay toegevoegd
 - Sketch Datalogger_05_LCD_RTC-DS3231_test_nulVoorTijdstip    toegevoegd  aan  Datalogger_09_SD_datalogger_RTC
 

 */



#include <SPI.h>                  // library voor SPI-interface
#include <SD.h>                   // library voor SD-kaartlezer
#include <Wire.h>                 // library voor I2C
#include <LiquidCrystal_I2C.h>    // library voor I2C-LCD
#include <DS3231.h>               // library voor RealTimeClock  (RTC) DS3231


// Set the LCD address to 0x27 for a 16 chars and 2 line display
// LiquidCrystal_I2C lcd(0x27, 16, 2);    // Gewoonlijk wordt 2 I2C adressen gebruikt: 0x27  of 0x3F
LiquidCrystal_I2C lcd(0x3F, 20, 4);       // Een groter display is handiger :  LCD2004 (2004 betekent 20 character horizontaal over 4 lijnen)
                                          //                                   LCD1602 (1602 betekent 16 character horizontaal over 2 lijnen)

DS3231 Clock;
bool Century=false;
bool h12;
bool PM;
byte ADay, AHour, AMinute, ASecond, ABits;
bool ADy, A12h, Apm;

const int chipSelect = 4;



void setup()   // ------------------------------------------ S E T U P ----------------------------------------------------------------------------
{
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) 
  {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  Serial.print("Initializing SD card...");

  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) 
  {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Serial.println("card initialized.");

  lcd.init();  // initialize the LCD

  // Turn on the blacklight and print a message.
  lcd.backlight();
  lcd.print("Klok:");
  delay(2000);
  
  // Start the I2C interface
  Wire.begin();
}

void loop()  // ------------------------------------------  L O O P ----------------------------------------------------------------------------
{


  // ------ datum en tijdstip in variabele --------
  String datum = "";
  if (Clock.getDate()<10)           // Als er 1 lijn achter IF-statement staat, dan mag je accolades weglaten (of op dezelfde regel schrijven
    datum += "0";                   // datum = datum + "0"  // is ook mogelijk
  datum = datum + String(Clock.getDate());
  datum += "/";
  if (Clock.getMonth(Century)<10) datum += "0";
  datum += String(Clock.getMonth(Century));
  datum += "/";
  datum += String(Clock.getYear());

  String tijdstip = "";
  if (Clock.getHour(h12, PM)<10) tijdstip += "0"; 
  tijdstip += String(Clock.getHour(h12, PM));
  tijdstip += ":";
  if (Clock.getMinute()<10)   tijdstip += "0"; 
  tijdstip += String(Clock.getMinute(), DEC);
  tijdstip += ":";
  if (Clock.getSecond()<10)  tijdstip += "0"; 
  tijdstip += String(Clock.getSecond(), DEC);


  
    
  lcd.setCursor(0, 0);
  lcd.print(datum);
  lcd.setCursor(10, 0);
  lcd.print(tijdstip);
 
 
  
  
  
  
  // ----- schrijf data met datum, tijdstip en Clocktemperatuur op SD kaart --------
  // Elke lijn begint met datum + tijdstip
  // make a string for assembling the data to log:
  // String dataString = "";
  String dataString = datum + ";" + tijdstip + ";" + String(Clock.getTemperature(), 2)+ ";";

  // read three sensors and append to the string:
  for (int analogPin = 0; analogPin < 3; analogPin++) {
    int sensor = analogRead(analogPin);
    dataString += String(sensor);
    if (analogPin < 2) {
      dataString += ";";
    }
  }

  // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
  File dataFile = SD.open("datalog01.csv", FILE_WRITE);

  // if the file is available, write to it:
  if (dataFile) {
    dataFile.println(dataString);
    dataFile.close();
    // print to the serial port too:
    Serial.println(dataString);
  }
  // if the file isn't open, pop up an error:
  else {
    Serial.println("error opening datalog01.csv");
  }

  delay(1000);
}









