/*
  SD card datalogger

 This example shows how to log data from three analog sensors
 to an SD card using the SD library.

 The circuit:
 * analog sensors on analog ins 0, 1, and 2
 * SD card attached to SPI bus as follows:
 ** MOSI - pin 11
 ** MISO - pin 12
 ** CLK - pin 13
 ** CS - pin 4 (for MKRZero SD: SDCARD_SS_PIN)

 created  24 Nov 2010
 modified 9 Apr 2012
 by Tom Igoe

 This example code is in the public domain.

 Jan Moeskops - 08/06/2019
 - Variabele teller toegevoegd
 - delay toegevoegd
 - Sketch Datalogger_05_LCD_RTC-DS3231_test_nulVoorTijdstip    toegevoegd  aan  Datalogger_09_SD_datalogger_RTC
 
Jan Moeskops - 09/06/2019
 - Sketch "Datalogger_07_LCD_RTC-DS3231_BH1750_DHT22.ino" toegevoegd
 - Niet elke seconde op SD kaart schrijven
 */



#include <SPI.h>                  // library voor SPI-interface
#include <SD.h>                   // library voor SD-kaartlezer
#include <Wire.h>                 // library voor I2C
#include <LiquidCrystal_I2C.h>    // library voor I2C-LCD
#include <DS3231.h>               // library voor RealTimeClock  (RTC) DS3231
#include <BH1750.h>               // library voor Luxmeter
#include <DHT.h>                  // library voor DHT22  temp en vochtigheidssensor

// Set the LCD address to 0x27 for a 16 chars and 2 line display
// LiquidCrystal_I2C lcd(0x27, 16, 2);    // Gewoonlijk wordt 2 I2C adressen gebruikt voor I2C LCDs: 0x27  of 0x3F   (met voorvoegsel 0x wordt hexadecimaal aangeduid)
LiquidCrystal_I2C lcd(0x3F, 20, 4);       // Een groter display is handiger :  LCD2004 (2004 betekent 20 character horizontaal over 4 lijnen)
                                          //                                   LCD1602 (1602 betekent 16 character horizontaal over 2 lijnen)

BH1750 lightMeter;                // intitieer BH1750


String bestandsnaam = "Data.csv";



#define DHTPIN 2     // Digital pin connected to the DHT sensor
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321
// Initialize DHT sensor.
// Note that older versions of this library took an optional third parameter to
// tweak the timings for faster processors.  This parameter is no longer needed
// as the current DHT reading algorithm adjusts itself to work on faster procs.
DHT dht(DHTPIN, DHTTYPE);

byte graden[8] = 
{
  0b01000,
  0b10100,
  0b10100,
  0b01000,
  0b00000,
  0b00000,
  0b00000,
  0b00000
};

#define DHTPIN 2     // Digital pin connected to the DHT sensor




DS3231 Clock;
bool Century=false;
bool h12;
bool PM;
byte ADay, AHour, AMinute, ASecond, ABits;
bool ADy, A12h, Apm;

const int chipSelect = 4;       // digit. aansluiting SD-card-reader

int titels = 50;                // om hoeveel aantal lijnen beschrijving in seriele monitor
int lijnteller = titels;

void setup()   // ------------------------------------------ S E T U P ----------------------------------------------------------------------------
{
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) 
  {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  Serial.print("Initializing SD card...");

  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) 
  {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Serial.println("card initialized.");
  Serial.print("Bestandsnaam = ");
  Serial.println(bestandsnaam);
  Serial.println("Datum    Tijd     TempI TempE Vocht W.Ind Licht   -------------");
  
  lcd.init();             // initialize the LCD
  lcd.backlight();        // Turn on the blacklight and print a message.
  lcd.setCursor(5, 1);    // Zet cursor op 6e plaats op 2e lijn

    //lcd.print("Datalogger");
  char titel[10] = {'D','a','t','a','l','o','g','g','e','r'};
  for(int i = 0; i < 10; i++)
  {
    lcd.print(titel[i]);
    delay(50);
  }

  lcd.createChar(0, graden);  // Speciaal character  °   (graden)
  
  delay(2000);
  lcd.clear();          // wis scherm
  
  Wire.begin();         // Start the I2C interface
  lightMeter.begin();   // Start the lightMeter
  dht.begin();          // Start DHT22

  
}

void loop()  // ------------------------------------------  L O O P ----------------------------------------------------------------------------
{


  // ------ datum en tijdstip in variabele omzetten --------
  String datum = "";
  if (Clock.getDate()<10)           // Als er 1 lijn achter IF-statement staat, dan mag je accolades weglaten (of op dezelfde regel schrijven
    datum += "0";                   // datum = datum + "0"  // is ook mogelijk
  datum = datum + String(Clock.getDate());
  datum += "/";
  if (Clock.getMonth(Century)<10) datum += "0";
  datum += String(Clock.getMonth(Century));
  datum += "/";
  datum += String(Clock.getYear());

  String tijdstip = "";
  if (Clock.getHour(h12, PM)<10) tijdstip += "0"; 
  tijdstip += String(Clock.getHour(h12, PM));
  tijdstip += ":";
  if (Clock.getMinute()<10)   tijdstip += "0"; 
  tijdstip += String(Clock.getMinute(), DEC);
  tijdstip += ":";
  if (Clock.getSecond()<10)  tijdstip += "0"; 
  tijdstip += String(Clock.getSecond(), DEC);

  float lux = lightMeter.readLightLevel();      // lichtsensor meet waarde ----------------------





 // Reading temperature or humidity takes about 250 milliseconds!     // DHT22 ----------------------
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  float h = dht.readHumidity();
  // Read temperature as Celsius (the default)
  float t = dht.readTemperature();
  // Read temperature as Fahrenheit (isFahrenheit = true)
  float f = dht.readTemperature(true);

  // Check if any reads failed and exit early (to try again).
  if (isnan(h) || isnan(t) || isnan(f)) {
    Serial.println(F("Failed to read from DHT sensor!"));
    return;
  }

  // Compute heat index in Fahrenheit (the default)
  float hif = dht.computeHeatIndex(f, h);
  // Compute heat index in Celsius (isFahreheit = false)
  float hic = dht.computeHeatIndex(t, h, false);

 


  // ------ Schrijf data naar LCD --------
    
  lcd.setCursor(0, 0);
  lcd.print(datum);           // 1. Datum
  lcd.setCursor(10, 0);
  lcd.print(tijdstip);        // 2. Tijdstip
  lcd.setCursor(0, 1);
  lcd.print(Clock.getTemperature(), 2);  // 3. Interne temperatuur (RTC)
  lcd.write(byte(0));             // when calling lcd.write() '0' must be cast as a byte  Print symbool graden
  lcd.print("C ");
  lcd.setCursor(10, 1);
  lcd.print(t);               // 4. Externe temperatuur
  lcd.write(byte(0));             // when calling lcd.write() '0' must be cast as a byte  Print symbool graden
  lcd.print("C ");
  lcd.setCursor(10, 2);
  lcd.print(hic);             // 6. Heat Index temperatuur
  lcd.write(byte(0)); // when calling lcd.write() '0' must be cast as a byte  Print symbool graden
  lcd.print("C ");
  lcd.setCursor(10, 3);
  lcd.print(h);               // 5. Luchtvochtigheid
  lcd.print(" %  ");
  lcd.setCursor(0, 3);
  //lcd.print("L:");
  lcd.print(lux);               // 7.  Lichtintensiteit Lux (BH1750)
  lcd.print(" lx");






  // ------ Schrijf data naar seriele monitor --------

  lijnteller--;
  if(lijnteller == 0) 
  {
    Serial.println("Datum    Tijd     TempI TempE Vocht W.Ind Licht   -------------");
    lijnteller = titels;
  }
  
  //Serial.print(titels-lijnteller+1);   // teller vooraan
  // Serial.print(" ");
  Serial.print(datum);                                  // 1.  datum
  //Serial.print("\t");
  Serial.print(" ");
  Serial.print(tijdstip);                               // 2.  tijdstip
  Serial.print(" ");
  Serial.print(Clock.getTemperature(), 2);              // 3.  Interne temperatuur (op RTC)
  Serial.print(" ");
  //Serial.print(F("%  Temperature: "));    
  Serial.print(t);                                      // 4.  Externe temperatuur (DHT22) 
  //Serial.print(F("°C "));
  Serial.print(" ");
  //Serial.print(F("Humidity: "));
  Serial.print(h);                                      // 5.  Luchtvochtigheid (DHT22) 
  //Serial.print(F("% "));
  Serial.print(" "); 
  Serial.print(hic);                                    // 6.  Warmte index berekend (DHT22) 
  //Serial.print(F("°C "));
  Serial.print(" "); 
  Serial.print(lux);                                    // 7.  Lichtintensiteit Lux (BH1750)
  //Serial.print(" lx  "); 
  // Serial.print(f);                                   // temp fahrenheit


  Serial.println("");
  
  // ----- schrijf data met datum, tijdstip en Clocktemperatuur op SD kaart --------

  //if ((Clock.getMinute() = 0 || Clock.getMinute() = 15 || || Clock.getMinute() = 30 || Clock.getMinute() = 45) && Clock.getSecond()== 0)   // elke 15 minuten
  if (Clock.getSecond()== 0)   // elke minuut
  {
      lcd.clear();
      lcd.setCursor(2, 1);
      lcd.print("Data op SD-kaart");
      lcd.setCursor(2, 2);
      for(int i = 0; i < 16; i++)
        {
          lcd.print(".");
          delay(100);
        }
      delay(300);
      lcd.clear();

      
      // Elke lijn begint met datum + tijdstip
      // make a string for assembling the data to log:
      // String dataString = "";
      String dataString = datum + ";" + tijdstip + ";";    // 1+2  datum + tijdstijp
      dataString += String(Clock.getTemperature(), 2);    // 3.  Interne temperatuur (op RTC)
      dataString += ";";
      dataString += t;                                    // 4.  Externe temperatuur (DHT22) 
      dataString += ";";
      dataString += h;                                    // 5.  Luchtvochtigheid (DHT22) 
      dataString += ";";
      dataString += hic;                                 // 6.  Warmte index berekend (DHT22) 
      dataString += ";";     
      dataString += lux;                                  // 7.  Lichtintensiteit Lux (BH1750)
      dataString += ";";

          
      // open the file. note that only one file can be open at a time,
      // so you have to close this one before opening another.
      File dataFile = SD.open(bestandsnaam, FILE_WRITE);
    
      // if the file is available, write to it:
      if (dataFile) 
      {
        dataFile.println(dataString);
        dataFile.close();
        // print to the serial port too:
        Serial.print(dataString);
        Serial.println("    --------- Bewaar op SD-kaart ---------");
      }
      // if the file isn't open, pop up an error:
      else 
      {
        Serial.print("error opening ");
        Serial.println(bestandsnaam);
      }
  }
  

  delay(1000);
}
