/*  ------     SD-kaart datalogger -------

  Deze sketch toont hoe je data kan loggen naar een csv-file op SD-kaart.
  Volgende sensoren/onderdelen zijn gebruikt:
  - I2C LCD2004 (om live mee te kijken)           https://www.tinytronics.nl/shop/nl/display/lcd/lcd-display-20*4-karakters-met-witte-tekst-en-blauwe-backlight-met-i2c-backpack
  - DHT22  (temp. luchtvochtigheid)               https://www.tinytronics.nl/shop/nl/sensoren/temperatuur-lucht-vochtigheid/dht22-thermometer-temperatuur-en-vochtigheids-sensor-module-met-kabels
  - RealTimeClock (RTC) DS3231                    https://www.tinytronics.nl/shop/nl/diversen/tijd/rtc-ds3231-met-at24c32-eeprom-incl.-batterij
  - Lichtintensiteitsensor (BH1750)               https://www.tinytronics.nl/shop/nl/sensoren/optisch/bh1750-16bit-digitale-i2c-licht-sensor-module
  - SD-kaart lezer  (SPI-aansluiting)             https://www.tinytronics.nl/shop/nl/diversen/opslag/microsd-kaart-adapter-module-3.3v-5v-met-level-shifter
  - Arduino Mega 2560  (extra geheugen is nodig)  https://www.tinytronics.nl/shop/nl/arduino/main-boards/mega-2560-r3-met-usb-kabel


  Aansluitingen op Arduino Mega 2560:
 * SPI  (groep van 6 pinnen) enkel voor SD-kaartlezer
      - pin 1 : MISO  (Master Input Slave Output)                  pin layout SPI  1  2
      - pin 2 : VCC = 5V                                                           3  4
      - pin 3 : SCK  (Serial Clock)                                                5  6
      - pin 4 : MOSI  (Master Output Slave Input)
      - pin 5 : niet verbonden
      - pin 6 : grond
      (referentie  https://www.arduino.cc/en/Reference/SPI )
  Pin4 voor CS (ChipSelect) van SD-kaartlezer
      
      
 * I2C : Voor BH1750, DS3231, I2C LCD2004
      - pin20 : SDA (Serial Data)
      - pin21 : SCL (Serial Clock)

 Pin2: data out op DHT22   (op DHT22 + = 5V,   - = GND )
      
 Deze sketch is een samenvoeging van voorbeeld programma's uit volgende library:
 - Arduino-LiquidCrystal-I2C-library-master -  HelloWorld
 - DHT_sensor_library  - DHTtester
 - DS3231  -  DS3231_test
 - BH1750-master - BH1750test
 - SD -  Datalogger + ListFiles + ReadWrite 
 - DS18B20 - DallasTemperature - Simple0000000000000000000000000000000000000000000

 Jan Moeskops - 08/06/2019
 - Variabele teller toegevoegd
 - delay toegevoegd
 - Sketch Datalogger_05_LCD_RTC-DS3231_test_nulVoorTijdstip    toegevoegd  aan  Datalogger_09_SD_datalogger_RTC
 
Jan Moeskops - 09/06/2019
 - Sketch "Datalogger_07_LCD_RTC-DS3231_BH1750_DHT22.ino" toegevoegd
 - Niet elke seconde op SD kaart schrijven
 - Seriele plotter of datalogger
 */


// ----- in te stellen -------------
int titels = 0;                // om hoeveel aantal lijnen beschrijving in seriele monitor  (0 = geen titels in seriële monitor)


bool SerielePlotter=false;      // Indien seriele plotter = True  dan kan je live grafiek tonen (zonder datum en tijdstip

String bestandsnaam = "Data1.csv";    // bestandsnaam waarin alle data geschreven wordt
// ---------------------------------

#include <SPI.h>                  // library voor SPI-interface
#include <SD.h>                   // library voor SD-kaartlezer
#include <Wire.h>                 // library voor I2C
#include <LiquidCrystal_I2C.h>    // library voor I2C-LCD
#include <DS3231.h>               // library voor RealTimeClock  (RTC) DS3231
#include <BH1750.h>               // library voor Luxmeter
#include <DHT.h>                  // library voor DHT22  temp en vochtigheidssensor
#include <OneWire.h>              // library voor DS18B20
#include <DallasTemperature.h>    // library voor DS18B20

// Set the LCD address to 0x27 for a 16 chars and 2 line display
// LiquidCrystal_I2C lcd(0x27, 16, 2);    // Gewoonlijk wordt 2 I2C adressen gebruikt voor I2C LCDs: 0x27  of 0x3F   (met voorvoegsel 0x wordt hexadecimaal aangeduid)
LiquidCrystal_I2C lcd(0x3F, 20, 4);       // Een groter display is handiger :  LCD2004 (2004 betekent 20 character horizontaal over 4 lijnen)
                                          //                                   LCD1602 (1602 betekent 16 character horizontaal over 2 lijnen)

BH1750 lightMeter;                // intitieer BH1750


#define ONE_WIRE_BUS 6                // Data wire DS18B20 is plugged into port 6 on the Arduino
OneWire oneWire(ONE_WIRE_BUS);        // Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
DallasTemperature sensors(&oneWire);  // Pass our oneWire reference to Dallas Temperature. 

#define DHTPIN 2     // Digital pin connected to the DHT sensor
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321
// Initialize DHT sensor.
// Note that older versions of this library took an optional third parameter to
// tweak the timings for faster processors.  This parameter is no longer needed
// as the current DHT reading algorithm adjusts itself to work on faster procs.
DHT dht(DHTPIN, DHTTYPE);

byte graden[8] = 
{
  0b01000,
  0b10100,
  0b10100,
  0b01000,
  0b00000,
  0b00000,
  0b00000,
  0b00000
};

#define DHTPIN 2     // Digital pin connected to the DHT sensor

int lijnteller = titels;


DS3231 Clock;
bool Century=false;
bool h12;
bool PM;
byte ADay, AHour, AMinute, ASecond, ABits;
bool ADy, A12h, Apm;

const int chipSelect = 4;       // digit. aansluiting SD-card-reader



void setup()   // ------------------------------------------ S E T U P ----------------------------------------------------------------------------
{
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) 
  {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  if(SerielePlotter)   
  {
    
      // see if the card is present and can be initialized:
      if (!SD.begin(chipSelect)) 
      {
          Serial.println("Card failed, or not present");
          // don't do anything more:
          return;
      }
      
  }
  else
  {
          Serial.print("Initializing SD card...");
        
          // see if the card is present and can be initialized:
          if (!SD.begin(chipSelect)) 
          {
            Serial.println("Card failed, or not present");
            // don't do anything more:
            return;
          }
          Serial.println("card initialized.");
          Serial.print("Bestandsnaam = ");
          Serial.println(bestandsnaam);
          Serial.println("Datum    Tijd     TempI TempE Vocht W.Ind Licht DS18B20");
  }
  

   
  sensors.begin();         // Start up the library DS18B20
  
  lcd.init();             // initialize the LCD
  lcd.backlight();        // Turn on the blacklight and print a message.
  lcd.setCursor(5, 1);    // Zet cursor op 6e plaats op 2e lijn

    //lcd.print("Datalogger");
  char titel[10] = {'D','a','t','a','l','o','g','g','e','r'};
  for(int i = 0; i < 10; i++)
  {
    lcd.print(titel[i]);
    delay(50);
  }

  lcd.createChar(0, graden);  // Speciaal character  °   (graden)
  
  delay(2000);
  lcd.clear();          // wis scherm
  
  Wire.begin();         // Start the I2C interface
  lightMeter.begin();   // Start the lightMeter
  dht.begin();          // Start DHT22

  
}

void loop()  // ------------------------------------------  L O O P ----------------------------------------------------------------------------
{


  // ------ datum en tijdstip in variabele omzetten --------
  String datum = "";
  if (Clock.getDate()<10)           // Als er 1 lijn achter IF-statement staat, dan mag je accolades weglaten (of op dezelfde regel schrijven
    datum += "0";                   // datum = datum + "0"  // is ook mogelijk
  datum = datum + String(Clock.getDate());
  datum += "/";
  if (Clock.getMonth(Century)<10) datum += "0";
  datum += String(Clock.getMonth(Century));
  datum += "/";
  datum += String(Clock.getYear());

  String tijdstip = "";
  if (Clock.getHour(h12, PM)<10) tijdstip += "0"; 
  tijdstip += String(Clock.getHour(h12, PM));
  tijdstip += ":";
  if (Clock.getMinute()<10)   tijdstip += "0"; 
  tijdstip += String(Clock.getMinute(), DEC);
  tijdstip += ":";
  if (Clock.getSecond()<10)  tijdstip += "0"; 
  tijdstip += String(Clock.getSecond(), DEC);

  float lux = lightMeter.readLightLevel();      // lichtsensor meet waarde ----------------------

  sensors.requestTemperatures(); // Send the command to get temperatures from DS18b20



 // Reading temperature or humidity takes about 250 milliseconds!     // DHT22 ----------------------
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  float h = dht.readHumidity();
  // Read temperature as Celsius (the default)
  float t = dht.readTemperature();
  // Read temperature as Fahrenheit (isFahrenheit = true)
  float f = dht.readTemperature(true);

  // Check if any reads failed and exit early (to try again).
  if (isnan(h) || isnan(t) || isnan(f)) {
    Serial.println(F("Failed to read from DHT sensor!"));
    return;
  }

  // Compute heat index in Fahrenheit (the default)
  float hif = dht.computeHeatIndex(f, h);
  // Compute heat index in Celsius (isFahreheit = false)
  float hic = dht.computeHeatIndex(t, h, false);

 


  // ------ Schrijf data naar LCD --------
    
  lcd.setCursor(0, 0);
  lcd.print(datum);           // 1. Datum
  lcd.setCursor(10, 0);
  lcd.print(tijdstip);        // 2. Tijdstip
  lcd.setCursor(0, 1);
  lcd.print(Clock.getTemperature(), 2);  // 3. Interne temperatuur (RTC)
  lcd.write(byte(0));             // when calling lcd.write() '0' must be cast as a byte  Print symbool graden
  lcd.print("C ");
  lcd.setCursor(10, 1);
  lcd.print(t);               // 4. Externe temperatuur
  lcd.write(byte(0));             // when calling lcd.write() '0' must be cast as a byte  Print symbool graden
  lcd.print("C ");
  lcd.setCursor(10, 2);
  lcd.print(hic);             // 6. Heat Index temperatuur
  lcd.write(byte(0)); // when calling lcd.write() '0' must be cast as a byte  Print symbool graden
  lcd.print("C ");
  lcd.setCursor(10, 3);
  lcd.print(h);               // 5. Luchtvochtigheid
  lcd.print(" %  ");
  lcd.setCursor(0, 3);
  //lcd.print("L:");
  lcd.print(lux);               // 7.  Lichtintensiteit Lux (BH1750)
  lcd.print(" lx");
  lcd.setCursor(0, 2);
  lcd.print(sensors.getTempCByIndex(0));             // 8. DS18B20 temperatuur
  lcd.write(byte(0)); // when calling lcd.write() '0' must be cast as a byte  Print symbool graden
  lcd.print("C ");




  // ------ Schrijf data naar seriele monitor --------

  lijnteller--;
  if(lijnteller == 0) 
  {
    Serial.println("Datum    Tijd     TempI TempE Vocht W.Ind Licht   -------------");
    lijnteller = titels;
  }



      // Elke lijn begint met datum + tijdstip
      // make a string for assembling the data to log:
      // String dataString = "";
      String dataString = datum + ";" + tijdstip + ";";    // 1+2  datum + tijdstijp
      dataString += String(Clock.getTemperature(), 2);    // 3.  Interne temperatuur (op RTC)
      dataString += ";";
      dataString += t;                                    // 4.  Externe temperatuur (DHT22) 
      dataString += ";";
      dataString += h;                                    // 5.  Luchtvochtigheid (DHT22) 
      dataString += ";";
      dataString += hic;                                 // 6.  Warmte index berekend (DHT22) 
      dataString += ";";     
      dataString += lux;                                  // 7.  Lichtintensiteit Lux (BH1750)
      dataString += ";";
      dataString += String(sensors.getTempCByIndex(0));    // 8. Temperatuur DS18B20


  if (SerielePlotter)   
  {
      Serial.print(Clock.getTemperature(), 2);              // 3.  Interne temperatuur (op RTC) -- blauw --  (lijnkleur in seriële plotter)
      Serial.print(" ");
      Serial.print(t);                                      // 4.  Externe temperatuur (DHT22)  -- rood -- 
      Serial.print(" ");
      Serial.print(h);                                      // 5.  Luchtvochtigheid (DHT22)     -- groen -- 
      Serial.print(" "); 
      Serial.print(hic);                                    // 6.  Warmte index berekend (DHT22) -- oranje  -- 
      Serial.print(" "); 
      Serial.print(lux);                                    // 7.  Lichtintensiteit Lux (BH1750) -- paars --
      Serial.print(" "); 
      Serial.println(sensors.getTempCByIndex(0));            // 8.  Temperatuur DS18B20
  }
  else
  {
      Serial.println(dataString);               // volledige dataString : Datum + tijdstip + interne temperatuur + externe temperatuur + luchtvochtigheid + warmte index + lichtintensiteit
  }
                             
 
  
  // ----- schrijf data met datum, tijdstip en Clocktemperatuur op SD kaart --------

  if ((Clock.getMinute() == 0 || Clock.getMinute() == 15 || Clock.getMinute() == 30 || Clock.getMinute() == 45) && Clock.getSecond()== 0)   // elke 15 minuten
  //if (Clock.getSecond()== 0)   // elke minuut
  {
      lcd.clear();
      lcd.setCursor(2, 1);
      lcd.print("Data op SD-kaart");
      lcd.setCursor(2, 2);
      for(int i = 0; i < 16; i++)
        {
          lcd.print(".");
          delay(50);
        }
      delay(200);
      lcd.clear();

      


          
      // open the file. note that only one file can be open at a time,
      // so you have to close this one before opening another.
      File dataFile = SD.open(bestandsnaam, FILE_WRITE);
    
      // if the file is available, write to it:
      if (dataFile) 
      {
        dataFile.println(dataString);
        dataFile.close();
      }
      // if the file isn't open, pop up an error:
      else 
      {
        Serial.print("error opening ");
        Serial.println(bestandsnaam);
      }
  }
  

  delay(1000);
}
